<?php
/**
 * This file contains endereco widget that put neccesary code to header.
 *
 * PHP Version 7
 *
 * @package   Endereco\OxidClient\Component\Widget
 * @author    Ilja Weber <ilja.weber@mobilemojo.de>
 * @copyright 2019 mobilemojo – Apps & eCommerce UG (haftungsbeschränkt) & Co. KG
 *            (https://www.mobilemojo.de/)
 * @license   http://opensource.org/licenses/gpl-3.0 GNU General Public License,
 *            version 3 (GPLv3)
 * @link      https://www.endereco.de/
 */

 /**
  * IncludeWidget
  *
  * Renders a piece of html in the header including all Endereco-SDK constructors
  * and creating variable configuration for them.
  *
  * PHP Version 7
  *
  * @package   Endereco\OxidClient\Component\Widget
  * @author    Ilja Weber <ilja.weber@mobilemojo.de>
  * @copyright 2019 mobilemojo – Apps & eCommerce UG (haftungsbeschränkt) & Co. KG
  *            (https://www.mobilemojo.de/)
  * @license   http://opensource.org/licenses/gpl-3.0 GNU General Public License,
  *            version 3 (GPLv3)
  * @link      https://www.endereco.de/
  */
class enderecoincludewidget extends oxWidget
{
    /**
     * @var string Widget template
     */
    protected $_sThisTemplate = 'enderecoincludewidget_flow.tpl';


    /**
     * Getter for protected property.
     *
     * @return string
     */
    public function getThisTemplate()
    {
        $aThemeMapping = array(
            'flow' => 'enderecoincludewidget_flow.tpl',
            'azure' => 'enderecoincludewidget_azure.tpl',
        );

        $oTheme = oxNew('oxtheme');
        $oTheme->load($oTheme->getActiveThemeId());

        $counter = 0;
        while (true) {
            $sThemeId = $oTheme->getId();

            if (isset($aThemeMapping[$sThemeId])) {
                return $aThemeMapping[$sThemeId];
            }

            // Get theme parent.
            $oTheme = $oTheme->getParent();

            if (null === $oTheme || (20 < $counter)) {
                break;
            }

            $counter++;
        }

        return $this->_sThisTemplate;
    }

    /**
     * Render
     *
     * @return string Template name.
     */
    public function render()
    {
        parent::render();

        if (session_status() == PHP_SESSION_NONE) {
            session_start();
        }
        unset($_SESSION['endereco']);

        $oConfig = $this->getConfig();
        $sOxId = oxRegistry::getConfig()->getRequestParameter('oxid');
        if (!$sOxId) {
            $sOxId = $oConfig->getShopId();
        }
        $this->_aViewData['enderecocstrs'] = array();

        $sql = "SELECT `OXVARNAME`, DECODE( `OXVARVALUE`, ? ) AS `OXVARVALUE` FROM `oxconfig` WHERE `OXSHOPID` = ? AND `OXMODULE` = 'module:enderecoclientox4'";
        $resultSet = oxDB::getDb()->getAll(
            $sql,
            array($oConfig->getConfigParam('sConfigKey'), $sOxId)
        );

        foreach ($resultSet as $result) {
            $this->_aViewData['enderecocstrs'][$result[0]] = $result[1];
        }

        return $this->getThisTemplate();
    }
}
